@@ -1,4 +1,3 @@
-import asyncio
 import copy
 from datetime import date, timedelta
 from typing import Dict, Tuple
@@ -17,15 +16,31 @@ from liualgotrader.models.optimizer import OptimizerRun
 from liualgotrader.models.portfolio import Portfolio
 from liualgotrader.trading.trader_factory import trader_factory
 
-AssetType
-
 est = timezone("America/New_York")
 
 
-def portfolio_return(
+async def load_trades_for_period(
+    env: str, from_date: date, to_date: date
+) -> pd.DataFrame:
+    query = f"""
+    SELECT client_time, symbol, operation, qty, price, algo_name
+    FROM 
+        new_trades as t, algo_run as a
+    WHERE 
+        t.algo_run_id = a.algo_run_id AND 
+        t.tstamp >= '{from_date}' AND 
+        t.tstamp < '{to_date}' AND
+        t.expire_tstamp is null AND
+        a.algo_env = '{env}'
+    ORDER BY symbol, tstamp
+    """
+    return await fetch_as_dataframe(query)
+
+
+async def portfolio_return(
     env: str, start_date: date
 ) -> Tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]:
-    df = load_trades_for_period(
+    df = await load_trades_for_period(
         env, start_date, date.today() + timedelta(days=1)
     )
 
@@ -33,7 +48,7 @@ def portfolio_return(
     table: Dict = {}
     invested_table: Dict = {}
     percentages_table: Dict = {}
-    for index, row in df.iterrows():
+    for _, row in df.iterrows():
         d = pd.Timestamp(pd.to_datetime(row["client_time"]).date(), tz=est)
         if d not in table:
             table[d] = {}
@@ -75,58 +90,12 @@ def portfolio_return(
     )
 
 
-def load_trades_for_period(
-    env: str, from_date: date, to_date: date
-) -> pd.DataFrame:
-    query = f"""
-    SELECT client_time, symbol, operation, qty, price, algo_name
-    FROM 
-        new_trades as t, algo_run as a
-    WHERE 
-        t.algo_run_id = a.algo_run_id AND 
-        t.tstamp >= '{from_date}' AND 
-        t.tstamp < '{to_date}' AND
-        t.expire_tstamp is null AND
-        a.algo_env = '{env}'
-    ORDER BY symbol, tstamp
-    """
-    loop = asyncio.get_event_loop()
-    return loop.run_until_complete(fetch_as_dataframe(query))
+async def load_trades(day: date, end_date: date = None) -> pd.DataFrame:
+    query = f"""\x1f    SELECT t.*, a.batch_id, a.algo_name\x1f    FROM \x1f    new_trades as t, algo_run as a\x1f    WHERE \x1f        t.algo_run_id = a.algo_run_id AND \x1f        t.tstamp >= '{day}' AND \x1f        t.tstamp < '{end_date or day + timedelta(days=1)}' AND\x1f        t.expire_tstamp is null \x1f    ORDER BY symbol, tstamp\x1f    """
+    return await fetch_as_dataframe(query)
 
 
-def load_trades(day: date, end_date: date = None) -> pd.DataFrame:
-    query = f"""
-    SELECT t.*, a.batch_id, a.algo_name
-    FROM 
-    new_trades as t, algo_run as a
-    WHERE 
-        t.algo_run_id = a.algo_run_id AND 
-        t.tstamp >= '{day}' AND 
-        t.tstamp < '{day + timedelta(days=1) if not end_date else end_date}' AND
-        t.expire_tstamp is null 
-    ORDER BY symbol, tstamp
-    """
-    loop = asyncio.get_event_loop()
-    return loop.run_until_complete(fetch_as_dataframe(query))
-
-
-def load_client_trades(day: date, end_date: date = None) -> pd.DataFrame:
-    query = f"""
-    SELECT t.*, a.batch_id, a.algo_name
-    FROM 
-    new_trades as t, algo_run as a
-    WHERE 
-        t.algo_run_id = a.algo_run_id AND 
-        t.tstamp >= '{day}' AND 
-        t.tstamp < '{day + timedelta(days=1) if not end_date else end_date}' AND
-        t.expire_tstamp is null 
-    ORDER BY symbol, tstamp
-    """
-    loop = asyncio.get_event_loop()
-    return loop.run_until_complete(fetch_as_dataframe(query))
-
-
-async def aload_trades_by_batch_id(batch_id: str) -> pd.DataFrame:
+async def load_trades_by_batch_id(batch_id: str) -> pd.DataFrame:
     query = f"""
         SELECT 
             t.*, a.batch_id, a.start_time, a.algo_name
@@ -145,15 +114,15 @@ async def aload_trades_by_batch_id(batch_id: str) -> pd.DataFrame:
             df["client_time"] = pd.to_datetime(df["client_time"])
     except Exception:
         tlog(
-            f"[Error] aload_trades_by_batch_id({batch_id}) can't convert 'client_time' column to datetime"
+            f"[Error] load_trades_by_batch_id({batch_id}) can't convert 'client_time' column to datetime"
         )
     return df
 
 
-async def aload_trades_by_portfolio_id(portfolio_id: str) -> pd.DataFrame:
+async def load_trades_by_portfolio(portfolio_id: str) -> pd.DataFrame:
     query = f"""
         SELECT 
-            t.*, a.batch_id, a.start_time, a.algo_name
+            distinct t.trade_id, t.algo_run_id, symbol, operation, qty, price, indicators, client_time, tstamp, stop_price, target_price, trade_fee, expire_tstamp, a.batch_id, a.start_time, a.algo_name
         FROM 
             new_trades as t, algo_run as a, portfolio_batch_ids as p
         WHERE 
@@ -171,38 +140,20 @@ async def aload_trades_by_portfolio_id(portfolio_id: str) -> pd.DataFrame:
             df["client_time"] = pd.to_datetime(df["client_time"])
     except Exception:
         tlog(
-            f"[Error] aload_trades_by_portfolio_id({portfolio_id}) can't convert 'client_time' column to datetime"
+            f"[Error] load_trades_by_portfolio({portfolio_id}) can't convert 'client_time' column to datetime"
         )
     return df
 
 
-def load_trades_by_batch_id(batch_id: str) -> pd.DataFrame:
-    loop = asyncio.get_event_loop()
-    return loop.run_until_complete(aload_trades_by_batch_id(batch_id))
+async def load_runs(day: date, end_date: date = None) -> pd.DataFrame:
+    query = f"""\x1f     SELECT * \x1f     FROM \x1f     algo_run as t\x1f     WHERE \x1f         start_time >= '{day}' AND \x1f         start_time < '{end_date or day + timedelta(days=1)}'\x1f     ORDER BY start_time\x1f     """
 
-
-def load_trades_by_portfolio(portfolio_id: str) -> pd.DataFrame:
-    loop = asyncio.get_event_loop()
-    return loop.run_until_complete(aload_trades_by_portfolio_id(portfolio_id))
-
-
-def load_runs(day: date, end_date: date = None) -> pd.DataFrame:
-    query = f"""
-     SELECT * 
-     FROM 
-     algo_run as t
-     WHERE 
-         start_time >= '{day}' AND 
-         start_time < '{day + timedelta(days=1) if not end_date else end_date}'
-     ORDER BY start_time
-     """
-    loop = asyncio.get_event_loop()
-    df = loop.run_until_complete(fetch_as_dataframe(query))
+    df = await fetch_as_dataframe(query)
     df.set_index("algo_run_id", inplace=True)
     return df
 
 
-def load_batch_list(day: date, env: str) -> pd.DataFrame:
+async def load_batch_list(day: date, env: str) -> pd.DataFrame:
     query = f"""
         SELECT DISTINCT a.batch_id, a.start_time
         FROM 
@@ -213,11 +164,10 @@ def load_batch_list(day: date, env: str) -> pd.DataFrame:
             t.tstamp < '{day + timedelta(days=1)}'AND
             t.expire_tstamp is null 
         """
-    loop = asyncio.get_event_loop()
-    return loop.run_until_complete(fetch_as_dataframe(query))
+    return await fetch_as_dataframe(query)
 
 
-def load_traded_symbols(batch_id: str) -> pd.DataFrame:
+async def load_traded_symbols(batch_id: str) -> pd.DataFrame:
     query = f"""
         SELECT 
             DISTINCT t.symbol
@@ -227,21 +177,7 @@ def load_traded_symbols(batch_id: str) -> pd.DataFrame:
             t.algo_run_id = a.algo_run_id AND 
             a.batch_id = '{batch_id}'
     """
-    loop = asyncio.get_event_loop()
-    return loop.run_until_complete(fetch_as_dataframe(query))
-
-
-def load_batch_symbols(batch_id: str) -> pd.DataFrame:
-    query = f"""
-        SELECT 
-            symbol
-        FROM 
-            trending_tickers
-        WHERE 
-            batch_id = '{batch_id}'
-    """
-    loop = asyncio.get_event_loop()
-    return loop.run_until_complete(fetch_as_dataframe(query))
+    return await fetch_as_dataframe(query)
 
 
 def calc_batch_revenue(
@@ -285,7 +221,7 @@ def count_trades(symbol, trades: pd.DataFrame, batch_id: str) -> int:
 
 
 def trades_analysis(trades: pd.DataFrame, batch_id: str) -> pd.DataFrame:
-    day_to_analyze = min(trades["client_time"].tolist())
+    min(trades["client_time"].tolist())
     trades_analytics = pd.DataFrame()
     trades["client_time"] = pd.to_datetime(trades["client_time"], utc=True)
     trades_analytics["symbol"] = trades.symbol.unique()
@@ -325,7 +261,7 @@ def symbol_trade_analytics(
 
     for _, row in symbol_df.iterrows():
         plt.scatter(
-            row["client_time"],  # .tz_convert("US/Eastern"),
+            row["client_time"],
             row["price"],
             c="g" if row["operation"] == "buy" else "r",
             s=100,
@@ -355,18 +291,23 @@ def calc_symbol_trades_returns(
 
         if t1 is not None and t2 is not None:
             for i in range(t1, t2):
-                value = (
-                    qty
-                    * data_loader[symbol].close[
-                        eastern.localize(
-                            daily_returns.index[i].to_pydatetime()
-                        ).replace(hour=9, minute=30, second=0, microsecond=0)
-                    ]
-                )
-                daily_returns.loc[
-                    daily_returns.index[i],
-                    "equity",
-                ] += value
+                try:
+                    value = (
+                        qty
+                        * data_loader[symbol].close[
+                            eastern.localize(
+                                daily_returns.index[i].to_pydatetime()
+                            ).replace(
+                                hour=9, minute=30, second=0, microsecond=0
+                            )
+                        ]
+                    )
+                    daily_returns.loc[
+                        daily_returns.index[i],
+                        "equity",
+                    ] += value
+                except Exception as e:
+                    tlog(f"Error during calc_symbol_trades_returns(): {e}")
 
         if row.operation == "buy":
             qty += row.qty
@@ -411,11 +352,10 @@ def calc_symbol_state(
     return qty, data_loader[symbol].close[-1]
 
 
-def calc_batch_returns(batch_id: str) -> pd.DataFrame:
-    loop = asyncio.get_event_loop()
-    portfolio = loop.run_until_complete(Portfolio.load_by_batch_id(batch_id))
+async def calc_batch_returns(batch_id: str) -> pd.DataFrame:
+    portfolio = await Portfolio.load_by_batch_id(batch_id)
     data_loader = DataLoader()
-    trades = load_trades_by_batch_id(batch_id)
+    trades = await load_trades_by_batch_id(batch_id)
 
     if trades.empty:
         return pd.DataFrame()
@@ -442,10 +382,11 @@ def calc_batch_returns(batch_id: str) -> pd.DataFrame:
     return pd.DataFrame(td, columns=["equity", "cash", "totals"])
 
 
-def compare_to_symbol_returns(portfolio_id: str, symbol: str) -> pd.DataFrame:
-
+async def compare_to_symbol_returns(
+    portfolio_id: str, symbol: str
+) -> pd.DataFrame:
     data_loader = DataLoader()
-    trades = load_trades_by_portfolio(portfolio_id)
+    trades = await load_trades_by_portfolio(portfolio_id)
     start_date = trades.client_time.min().date()
     end_date = trades.client_time.max().date()
     trader = trader_factory()
@@ -460,9 +401,8 @@ def compare_to_symbol_returns(portfolio_id: str, symbol: str) -> pd.DataFrame:
     return td[symbol]
 
 
-def get_cash(account_id: int, initial_cash: float) -> pd.DataFrame:
-    loop = asyncio.get_event_loop()
-    df = loop.run_until_complete(Accounts.get_transactions(account_id))
+async def get_cash(account_id: int, initial_cash: float) -> pd.DataFrame:
+    df = await Accounts.get_transactions(account_id)
     df = df.groupby(df.index.date).sum()
     df.iloc[0].amount += initial_cash
     r = pd.date_range(start=df.index.min(), end=df.index.max(), name="date")
@@ -472,14 +412,11 @@ def get_cash(account_id: int, initial_cash: float) -> pd.DataFrame:
     return df
 
 
-def calc_portfolio_returns(portfolio_id: str) -> pd.DataFrame:
-    loop = asyncio.get_event_loop()
-    portfolio = loop.run_until_complete(
-        Portfolio.load_by_portfolio_id(portfolio_id)
-    )
+async def calc_portfolio_returns(portfolio_id: str) -> pd.DataFrame:
+    portfolio = await Portfolio.load_by_portfolio_id(portfolio_id)
 
     data_loader = DataLoader()
-    trades = load_trades_by_portfolio(portfolio_id)
+    trades = await load_trades_by_portfolio(portfolio_id)
 
     if trades.empty:
         return pd.DataFrame()
@@ -494,7 +431,7 @@ def calc_portfolio_returns(portfolio_id: str) -> pd.DataFrame:
 
     td["equity"] = 0.0
 
-    cash_df = get_cash(portfolio.account_id, portfolio.portfolio_size)
+    cash_df = await get_cash(portfolio.account_id, portfolio.portfolio_size)
 
     symbols = trades.symbol.unique().tolist()
     for i in tqdm(
@@ -511,17 +448,16 @@ def calc_portfolio_returns(portfolio_id: str) -> pd.DataFrame:
     return pd.DataFrame(td, columns=["equity", "cash", "totals"])
 
 
-def calc_hyperparameters_analysis(optimizer_run_id: str) -> pd.DataFrame:
-    loop = asyncio.get_event_loop()
-
-    portfolio_ids_parameters = loop.run_until_complete(
-        OptimizerRun.get_portfolio_ids_parameters(optimizer_run_id)
+async def calc_hyperparameters_analysis(optimizer_run_id: str) -> pd.DataFrame:
+    portfolio_ids_parameters = await OptimizerRun.get_portfolio_ids_parameters(
+        optimizer_run_id
     )
+
     portfolio_ids, hypers = list(map(list, zip(*portfolio_ids_parameters)))
     df = None
     if len(portfolio_ids):
         for i in tqdm(range(len(portfolio_ids)), desc="Loading Portfolios"):
-            _df = calc_portfolio_returns(portfolio_ids[i])
+            _df = await calc_portfolio_returns(portfolio_ids[i])
 
             if _df.empty:
                 continue
@@ -535,13 +471,10 @@ def calc_hyperparameters_analysis(optimizer_run_id: str) -> pd.DataFrame:
     return df
 
 
-def get_portfolio_equity(portfolio_id: str) -> pd.DataFrame:
-    loop = asyncio.get_event_loop()
-    portfolio = loop.run_until_complete(
-        Portfolio.load_by_portfolio_id(portfolio_id)
-    )
+async def get_portfolio_equity(portfolio_id: str) -> pd.DataFrame:
+    await Portfolio.load_by_portfolio_id(portfolio_id)
     data_loader = DataLoader()
-    trades = load_trades_by_portfolio(portfolio_id)
+    trades = await load_trades_by_portfolio(portfolio_id)
 
     symbols = trades.symbol.unique().tolist()
     rows = []
@@ -572,12 +505,6 @@ def get_portfolio_equity(portfolio_id: str) -> pd.DataFrame:
     return df.loc[df.qty > 0].round(2)
 
 
-def get_portfolio_cash(portfolio_id: str) -> pd.DataFrame:
-    loop = asyncio.get_event_loop()
-    portfolio = loop.run_until_complete(
-        Portfolio.load_by_portfolio_id(portfolio_id)
-    )
-
-    return loop.run_until_complete(
-        Accounts.get_transactions(portfolio.account_id)
-    ).round(2)
+async def get_portfolio_cash(portfolio_id: str) -> pd.DataFrame:
+    portfolio = await Portfolio.load_by_portfolio_id(portfolio_id)
+    return (await Accounts.get_transactions(portfolio.account_id)).round(2)
