@@ -1,15 +1,16 @@
-import io
-import time
-from datetime import date, datetime, timedelta
-from typing import Dict, List
+from datetime import date, datetime
+from typing import Dict, List, Set
 
+import numpy as np
 import pandas as pd
 import pandas_market_calendars
+import pytz
 import requests
 from pandas import DataFrame as df
 
 from liualgotrader.common import config
 from liualgotrader.common.data_loader import DataLoader  # type: ignore
+from liualgotrader.common.data_loader import m_and_a_data  # type: ignore
 from liualgotrader.common.tlog import tlog
 from liualgotrader.common.types import TimeScale
 from liualgotrader.fincalcs.vwap import add_daily_vwap
@@ -17,9 +18,8 @@ from liualgotrader.fincalcs.vwap import add_daily_vwap
 volume_today: Dict[str, int] = {}
 quotes: Dict[str, df] = {}
 
-m_and_a_data = pd.read_csv(
-    "https://raw.githubusercontent.com/amor71/LiuAlgoTrader/master/database/market_m_a_data.csv"
-).set_index("date")
+NY = "America/New_York"
+nytz = pytz.timezone(NY)
 
 
 def get_symbol_data(
@@ -114,20 +114,56 @@ async def get_market_industries() -> List[str]:
         return [record[0] for record in records if record[0]]
 
 
-async def sp500_historical_constituents(date):
+async def sp500_historical_constituents(date: str):
     tlog(f"loading sp500 constituents for {date}")
     table = pd.read_html(
         "https://en.wikipedia.org/wiki/List_of_S%26P_500_companies"
     )
-    symbols = table[0].Symbol.to_list()
+    symbols: List = table[0].Symbol.to_list()
     changes = table[1]
+
     changes["date"] = changes.Date.apply(
         lambda x: datetime.strptime(x[0], "%B %d, %Y"), axis=1
     )
+    m_and_a_data.index = m_and_a_data.index.astype("datetime64[ns]", copy=True)
+
+    adjusted_symbols = m_and_a_data.loc[date < m_and_a_data.index, "to_symbol"]
+    print("adjusted_symbols", adjusted_symbols)
     changes = changes.loc[changes.date > date]
-    added = changes.Added.dropna().Ticker.to_list()
-    removed = changes.Removed.dropna().Ticker.to_list()
-    return list(set(symbols) - set(added)) + removed
+
+    unadusted: Set = set(symbols)
+    while True:
+        no_changes = True
+        for _, row in changes.iterrows():
+            if not row.Added.dropna().empty and row.Added.Ticker in unadusted:
+                unadusted.remove(row.Added.Ticker)
+                no_changes = False
+            if (
+                not row.Removed.dropna().empty
+                and row.Removed.Ticker in unadusted
+            ):
+                unadusted.add(row.Removed.Ticker)
+                no_changes = False
+
+        if no_changes:
+            break
+
+    while True:
+        no_changes = True
+        for symbol in adjusted_symbols:
+            if symbol in unadusted:
+                unadusted.add(
+                    m_and_a_data.loc[
+                        m_and_a_data.to_symbol == symbol, "from_symbol"
+                    ].item()
+                )
+                unadusted.remove(symbol)
+                no_changes = False
+
+        if no_changes:
+            break
+
+    return list(unadusted)
 
 
 async def get_trading_holidays() -> List[str]:
