@@ -2,17 +2,16 @@
 Execute Strategies on streaming data received from the Producer
 """
 import asyncio
-import concurrent.futures
+import math
 import os
-import sys
 from datetime import datetime, timedelta
+from multiprocessing import Queue
 from queue import Empty
 from random import randint
 from typing import Any, Dict, List, Optional
 
 import pandas as pd
 import pygit2
-from mnqueues import MNQueue
 from pandas import DataFrame as df
 from pytz import timezone
 
@@ -20,9 +19,7 @@ from liualgotrader.analytics.analysis import load_trades_by_portfolio
 from liualgotrader.common import config, market_data, trading_data
 from liualgotrader.common.data_loader import DataLoader  # type: ignore
 from liualgotrader.common.database import create_db_connection
-from liualgotrader.common.decorators import trace  # type: ignore
 from liualgotrader.common.tlog import tlog, tlog_exception
-from liualgotrader.common.tracer import trace_elapsed_metrics  # type: ignore
 from liualgotrader.common.types import Order, Trade
 from liualgotrader.fincalcs.data_conditions import QUOTE_SKIP_CONDITIONS
 from liualgotrader.models.new_trades import NewTrade
@@ -164,7 +161,7 @@ async def periodic_runner(data_loader: DataLoader, trader: Trader) -> None:
 
                 tasks.append(
                     asyncio.create_task(
-                        trace({})(do_strategy_all)(
+                        do_strategy_all(
                             trader=trader,
                             data_loader=data_loader,
                             strategy=s,
@@ -192,11 +189,6 @@ async def should_cancel_order(order: Order, market_clock: datetime) -> bool:
     # Make sure the order's not too old
     submitted_at = order.submitted_at.astimezone(market_clock.tzinfo)
     order_lifetime = market_clock - submitted_at
-
-    if config.debug_enabled:
-        tlog(
-            f"should_cancel_order submitted_at:{submitted_at}, order_lifetime:{order_lifetime}"
-        )
     return (
         market_clock > submitted_at
         and order_lifetime.total_seconds() // 60 >= 1
@@ -224,7 +216,7 @@ async def save(
 
     await db_trade.save(
         config.db_conn_pool,
-        str(now),
+        now,
         trading_data.stop_prices[symbol]
         if symbol in trading_data.stop_prices
         else 0.0,
@@ -292,7 +284,7 @@ async def update_partially_filled_order(
         side.name,
         filled_avg_price,
         indicators,
-        updated_at,
+        str(updated_at),
         trade_fee,
     )
 
@@ -364,9 +356,7 @@ async def handle_trade_update_for_order(trade: Trade) -> bool:
 
 
 async def handle_trade_update_wo_order(trade: Trade) -> bool:
-    symbol = trade.symbol.lower()
-    event = trade.event
-    # tlog(f"trade update without order for {symbol} data={trade} with event {event}")
+    tlog(f"trade update without order for {trade}")
     return True
 
 
@@ -436,6 +426,7 @@ async def handle_quote(data: Dict) -> bool:
 async def aggregate_bar_data(
     data_loader: DataLoader, data: Dict, ts: pd.Timestamp, carrier=None
 ) -> None:
+    ts = ts.replace(second=0)
     symbol = data["symbol"].lower()
 
     if not data_loader.exist(symbol):
@@ -456,7 +447,7 @@ async def aggregate_bar_data(
             data["volume"],
             data["vwap"],
             data["average"],
-            data["count"],
+            int(data["count"]),
         ]
     else:
         new_data = [
@@ -466,9 +457,12 @@ async def aggregate_bar_data(
             data["close"],
             current["volume"] + data["volume"],
             data["vwap"] or current["vwap"],
-            data["average"] or current["average"],
-            data["count"] + current["count"],
+            current["average"]
+            if math.isnan(data["average"])
+            else data["average"],
+            int(data["count"] + current["count"]),
         ]
+
     try:
         data_loader[symbol].loc[ts] = new_data
     except ValueError:
@@ -511,12 +505,12 @@ async def order_inflight(
                 )
                 await update_filled_order(
                     symbol=symbol,
-                    strategy=trading_data.open_order_strategy[symbol],
-                    filled_avg_price=filled_price,
-                    filled_qty=filled_qty,
-                    side=existing_order.side,
-                    updated_at=existing_order.submitted_at,
-                    trade_fee=trade_fee,
+                    strategy=trading_data.open_order_strategy[symbol],  # type: ignore
+                    filled_avg_price=filled_price,  # type: ignore
+                    filled_qty=filled_qty,  # type: ignore
+                    side=existing_order.side,  # type: ignore
+                    updated_at=existing_order.submitted_at,  # type: ignore
+                    trade_fee=trade_fee,  # type: ignore
                 )
             elif order_status == Order.EventType.partial_fill:
                 tlog(
@@ -525,11 +519,11 @@ async def order_inflight(
                 await update_partially_filled_order(
                     symbol=symbol,
                     strategy=trading_data.open_order_strategy[symbol],
-                    side=existing_order.side,
-                    filled_avg_price=filled_price,
-                    filled_qty=filled_qty,
+                    side=existing_order.side,  # type: ignore
+                    filled_avg_price=filled_price,  # type: ignore
+                    filled_qty=filled_qty,  # type: ignore
                     updated_at=existing_order.submitted_at,
-                    trade_fee=trade_fee,
+                    trade_fee=trade_fee,  # type: ignore
                 )
             else:
                 # Cancel it so we can try again for a fill
@@ -633,10 +627,6 @@ async def do_strategy(
     portfolio_value: Optional[float],
     carrier=None,
 ) -> bool:
-    if not trader.is_market_open(datetime.now(nyc)):
-        tlog(f"market closed, can't execute {symbol} @ {strategy.name}")
-        return False
-
     do, what = await strategy.run(
         symbol=symbol,
         shortable=shortable,
@@ -681,7 +671,7 @@ async def do_strategies(
     strategies = await _filter_strategies(symbol)
     for s in strategies:
         try:
-            if not await trace(carrier)(do_strategy)(
+            if not await do_strategy(
                 strategy=s,
                 symbol=symbol,
                 shortable=shortable[symbol],
@@ -715,15 +705,15 @@ async def handle_aggregate(
 ) -> bool:
     symbol = symbol.lower()
 
-    if config.debug_enabled:
-        tlog(f"handle_aggregate {symbol}")
+    await aggregate_bar_data(data_loader, data, ts)
+
     # Next, check for existing orders for the stock
     if symbol in trading_data.open_orders and await order_inflight(
         symbol, trading_data.open_orders[symbol.lower()], ts, trader
     ):
         return True
 
-    await trace(carrier)(do_strategies)(
+    await do_strategies(
         trader=trader,
         data_loader=data_loader,
         symbol=symbol,
@@ -748,12 +738,8 @@ async def handle_data_queue_msg(
     symbol = data["symbol"].lower()
     shortable[symbol] = True  # TODO revisit collecting 'is shortable' data
 
-    await trace(carrier)(aggregate_bar_data)(data_loader, data, ts)
-
     # timezone("America/New_York")
     time_diff = datetime.now(tz=data["timestamp"].tz) - data["timestamp"]
-    if config.trace_enabled:
-        trace_elapsed_metrics("DL", time_diff)
 
     if data["EV"] != "AM" and time_diff > timedelta(seconds=15):
         if randint(1, 100) == 1:  # nosec
@@ -768,7 +754,7 @@ async def handle_data_queue_msg(
         return True
 
     time_tick[symbol] = data["timestamp"].replace(microsecond=0, nanosecond=0)
-    await trace(carrier)(handle_aggregate)(
+    await handle_aggregate(
         trader=trader,
         data_loader=data_loader,
         symbol=symbol,
@@ -781,7 +767,7 @@ async def handle_data_queue_msg(
 
 
 async def queue_consumer(
-    batch_id: str, queue: MNQueue, data_loader: DataLoader, trader: Trader
+    batch_id: str, queue: Queue, data_loader: DataLoader, trader: Trader
 ) -> None:
     tlog("queue_consumer() starting")
     try:
@@ -790,7 +776,8 @@ async def queue_consumer(
                 data = queue.get(timeout=2)
                 if data["EV"] == "trade_update":
                     tlog(f"received trade_update: {data}")
-                    await handle_trade_update(Trade(**data["trade"]))
+                    t = Trade(**data["trade"])
+                    await handle_trade_update(t)
                 elif data["EV"] == "new_strategy":
                     tlog(f"received new_strategy: {data}")
                     await handle_new_strategy(
@@ -800,9 +787,7 @@ async def queue_consumer(
                         parameters=data["parameters"],
                     )
                 else:
-                    await trace({})(handle_data_queue_msg)(
-                        data, trader, data_loader
-                    )
+                    await handle_data_queue_msg(data, trader, data_loader)
 
             except Empty:
                 await asyncio.sleep(0)
@@ -850,11 +835,21 @@ async def create_strategies_from_file(
         if s:
             strategy_list.append(s)
 
+        if "portfolio_id" in strategies_conf[strategy_name]:
+            positions = await load_symbol_position(
+                strategies_conf[strategy_name]["portfolio_id"]
+            )
+            for symbol, qty in positions.items():
+                trading_data.positions[symbol] = (
+                    trading_data.positions.get(symbol, 0.0) + qty
+                )
+            tlog(f"Loaded {positions} positions for {strategy_name}")
+
     return strategy_list
 
 
 async def load_symbol_position(portfolio_id: str) -> Dict[str, float]:
-    trades = load_trades_by_portfolio(portfolio_id)
+    trades = await load_trades_by_portfolio(portfolio_id)
 
     if not len(trades):
         return {}
@@ -870,11 +865,7 @@ async def load_symbol_position(portfolio_id: str) -> Dict[str, float]:
     )
     new_df = new_df.loc[new_df.qty != 0]
 
-    rc_dict: Dict[str, float] = {}
-    for _, row in new_df.iterrows():
-        rc_dict[row.symbol] = float(row.qty)
-
-    return rc_dict
+    return {row.symbol: float(row.qty) for _, row in new_df.iterrows()}
 
 
 async def create_strategies_from_db(
@@ -902,7 +893,7 @@ async def create_strategies_from_db(
             positions = await load_symbol_position(
                 trade_plan_entry.portfolio_id
             )
-            tlog(f"Loaded {len(positions)} positions for strategy_name")
+            tlog(f"Loaded {len(positions)} positions for {strategy_name}")
             for symbol, qty in positions.items():
                 trading_data.positions[symbol] = (
                     trading_data.positions.get(symbol, 0.0) + qty
@@ -928,9 +919,10 @@ async def handle_new_strategy(
 
 
 async def consumer_async_main(
-    queue: MNQueue,
+    queue: Queue,
     unique_id: str,
     strategies_conf: Dict,
+    file_only: bool,
 ):
     await create_db_connection(str(config.dsn))
     data_loader = DataLoader()
@@ -943,12 +935,13 @@ async def consumer_async_main(
         data_loader=data_loader,
         strategies_conf=strategies_conf,
     )
-
-    trading_data.strategies += await create_strategies_from_db(
-        batch_id=unique_id,
-        trader=trader,
-        data_loader=data_loader,
-    )
+    tlog(f"loaded strategies: {trading_data.strategies}")
+    if not file_only:
+        trading_data.strategies += await create_strategies_from_db(
+            batch_id=unique_id,
+            trader=trader,
+            data_loader=data_loader,
+        )
 
     queue_consumer_task = asyncio.create_task(
         queue_consumer(unique_id, queue, data_loader, trader)
@@ -973,7 +966,7 @@ async def consumer_async_main(
 
 
 def consumer_main(
-    queue: MNQueue,
+    queue: Queue,
     unique_id: str,
     conf: Dict,
 ) -> None:
@@ -993,7 +986,14 @@ def consumer_main(
         config.risk = conf["risk"]
 
     try:
-        asyncio.run(consumer_async_main(queue, unique_id, conf["strategies"]))
+        asyncio.run(
+            consumer_async_main(
+                queue,
+                unique_id,
+                conf["strategies"],
+                conf.get("file_only", False),
+            ),
+        )
     except KeyboardInterrupt:
         tlog("consumer_main() - Caught KeyboardInterrupt")
 
