@@ -1,9 +1,7 @@
 import asyncio
 import inspect
-import sys
 import traceback
 import uuid
-from calendar import Calendar
 from datetime import date, datetime, timedelta
 from typing import Dict, List, Optional
 
@@ -96,9 +94,7 @@ async def do_scanners(
         new_symbols = await scanner.run(back_time=now)
         target_strategy_name = scanner.target_strategy_name
 
-        target_strategy_name = (
-            "_all" if not target_strategy_name else target_strategy_name
-        )
+        target_strategy_name = target_strategy_name or "_all"
 
         symbols[target_strategy_name] = new_symbols
 
@@ -380,22 +376,18 @@ async def backtest_day(
         for symbol in prefetch:
             tlog(f"Prefetch data for {symbol}@{day_start}-{day_end}")
 
-            # try:
             data_loader[symbol][day_start:day_end]
             prefetched.append(symbol)
-            # except ValueError:
-            #    tlog(f"no data for {symbol} on {day_start}")
 
         for strategy in strategies:
             try:
-                trading_data.positions.update(
-                    {
-                        symbol: 0
-                        for symbol in symbols.get(strategy.name, [])
-                        + symbols.get("_all", [])
-                        if symbol not in trading_data.positions
-                    }
-                )
+                trading_data.positions |= {
+                    symbol: 0
+                    for symbol in symbols.get(strategy.name, [])
+                    + symbols.get("_all", [])
+                    if symbol not in trading_data.positions
+                }
+
             except TypeError as e:
                 if config.debug_enabled:
                     tlog_exception("backtest_day")
